from zope.app.file.image import Image as ImageBase
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

class Image(ImageBase, DefaultDublinCoreImpl):
    """
    We need our own Image class here to mix in Dublin Core, since the
    zope image object uses zope 3 machinery to attribute dublin core
    properties which doesn't work here. Or at least I can't get one of
    these image objects to adapt to
    zope.dublincore.interfaces.IWriteZopeDublinCore
    """
    def __init__(self, data=''):
        ImageBase.__init__(self, data)
        DefaultDublinCoreImpl.__init__(self)



from OFS.Image import Image

from zope.app.form.browser.widget import DisplayWidget
from zope.app.form.browser.textwidgets import FileWidget
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile


class ImageWidget(FileWidget):
    """
    The standard FileWidget returns a string instead of an IFile inst,
    which means it will always fail schema validation in formlib.
    """

    template = ViewPageTemplateFile('inputwidget.pt')
    displayWidth = 30

    def __call__(self):
        value=self._getFormValue() or None
        return self.template(name=self.context.__name__, value=value)

    def _toFieldValue(self, input):
        value=super(ImageWidget, self)._toFieldValue(input)
        if value is not self.context.missing_value:
            value=Image('image','image', value)
        return value

    def hasInput(self):
        return ((self.name+".used" in self.request.form)
                or
                (self.name in self.request.form)
                ) and not self.request.form.get(self.name+".nochange", '')



from zope.interface import Interface

from zope.interface import implements
from zope.component import getMultiAdapter
from zope import schema
from zope.formlib import form

from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.portlets import base
from plone.memoize.instance import memoize
from plone.app.vocabularies.catalog import SearchableTextSourceBinder
from plone.app.form.widgets.uberselectionwidget import UberSelectionWidget

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from ely.portlets.image import ImagePortletMessageFactory as _
from ely.portlets.image.widget import ImageWidget

class IImagePortlet(IPortletDataProvider):
    """A portlet

    It inherits from IPortletDataProvider because for this portlet, the
    data that is being rendered and the portlet assignment itself are the
    same.
    """

    header = schema.TextLine(title=_(u"Title"),
                             description=_(u"Title for the image"),
                             required=False)

    display_title = schema.Bool(title=_(u'Display Title'),
                                description=_(u"Select to display the title"),
                                required=False,
                                default=True)

    caption = schema.TextLine(title=_(u"Caption"),
                              description=_(u"Caption for the image"),
                              required=False)

    internal_target = schema.Choice(title=_(u"Internal Target"),
                            description=_(u"Find an internal target for this image to link to"),
                            required=False,
                            source=SearchableTextSourceBinder({}))

    image = schema.Field(title=u'Image field',
                         description=u"Add or replace image for the portlet",
                         required=False)

    external_url = schema.TextLine(title=_(u"External url"),
                              description=_(u"If target object is not selected the image will link to this url"),
                              required=False)


class Assignment(base.Assignment):
    """Portlet assignment.

    This is what is actually managed through the portlets UI and associated
    with columns.
    """

    implements(IImagePortlet)

    header = u''
    display_title = True
    caption = u''
    image = None
    assignment_context_path = None
    internal_target = None
    external_url = ''

    def __init__(self,
                 assignment_context_path=None,
                 image=None,
                 header=None,
                 display_title=True,
                 caption=u'',
                 internal_target=None,
                 external_url=''):
        self.image = image
        self.assignment_context_path = assignment_context_path
        self.internal_target = internal_target
        self.header = header
        self.caption = caption
        self.external_url = external_url

    @property
    def title(self):
        if self.header:
            return self.header
        else:
            return 'Image Portlet'


class Renderer(base.Renderer):
    """Portlet renderer.
    """

    render = ViewPageTemplateFile('imageportlet.pt')

    @property
    @memoize
    def image_tag(self):
        if self.data.image:
            state=getMultiAdapter((self.context, self.request),
                                  name="plone_portal_state")
            portal=state.portal()
            assignment_url = \
                    portal.unrestrictedTraverse(
                self.data.assignment_context_path).absolute_url()
            width = self.data.image.width
            height = self.data.image.height
            return "<img src='%s/%s/@@image' width='%s' height='%s'/>" % \
                   (assignment_url,
                    self.data.__name__,
                    str(width),
                    str(height))
        return None

    @property
    @memoize
    def linked_object(self):
        state=getMultiAdapter((self.context, self.request),
                              name="plone_portal_state")
        portal=state.portal()
        object_path = self.data.internal_target
        # it feels insane that i need to do manual strippping of the
        # first slash in this string. I must be doing something wrong
        # please make this bit more sane

        if object_path is None or len(object_path)==0:
            return None
        # The portal root is never a target

        if object_path[0]=='/':
            object_path = object_path[1:]
        return portal.restrictedTraverse(object_path, default=None)

    @property
    def link_url(self):
        linked_object = self.linked_object
        if linked_object:
            return linked_object.absolute_url()
        else:
            return self.data.external_url

class AddForm(base.AddForm):
    """Portlet add form.
    """
    form_fields = form.Fields(IImagePortlet)
    form_fields['image'].custom_widget = ImageWidget
    form_fields['internal_target'].custom_widget = UberSelectionWidget

    def create(self, data):
        assignment_context_path = \
                    '/'.join(self.context.__parent__.getPhysicalPath())
        return Assignment(assignment_context_path=assignment_context_path,
                          **data)

class EditForm(base.EditForm):
    """Portlet edit form.
    """
    form_fields = form.Fields(IImagePortlet)
    form_fields['image'].custom_widget = ImageWidget
    form_fields['internal_target'].custom_widget = UberSelectionWidget



from Acquisition import aq_inner
from zope.interface import implements

from Products.Five.browser import BrowserView

from ely.portlets.image.interfaces import IImagePortletImageView

class ImageView(BrowserView):
    '''
    View the image field of the image portlet. We steal header details
    from zope.app.file.browser.file and adapt it to use the dublin
    core implementation that the Image object here has.
    '''

    implements(IImagePortletImageView)

    def __call__(self):
        context = aq_inner(self.context)
        image = context.image
        return image.index_html(self.request, self.request.response)



import os
from OFS.Image import Image
from zope.component import getUtility, getMultiAdapter

from plone.portlets.interfaces import IPortletType
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletAssignment
from plone.portlets.interfaces import IPortletDataProvider
from plone.portlets.interfaces import IPortletRenderer

from plone.app.portlets.storage import PortletAssignmentMapping

from ely.portlets.image import imageportlet

from ely.portlets.image.tests.base import TestCase

def _test_image_file():
    """
    get test image object
    """
    here = os.path.dirname(os.path.realpath(__file__))
    path =  os.path.join(here, 'test_image.jpg')
    return file(path)

def _test_image_object():
    """
    create the image object type that the formlib field creates
    for the uploaded image.
    """
    image = Image('image', 'image', _test_image_file())
    return image

def test_assignment_data():
    data = {
        'header':u'Test Image Portlet',
        'caption':u'some kind of image',
        'image':_test_image_object(),
        'assignment_context_path':'/plone/++contextportlets++plone.rightcolumn',
        'internal_target':'front-page',
        }
    return data

class TestPortlet(TestCase):

    def afterSetUp(self):
        self.setRoles(('Manager',))


    def test_portlet_type_registered(self):
        portlet = getUtility(IPortletType, name='ely.portlets.image.ImagePortlet')
        self.assertEquals(portlet.addview, 'ely.portlets.image.ImagePortlet')

    def test_interfaces(self):
        portlet = imageportlet.Assignment(**test_assignment_data())
        self.failUnless(IPortletAssignment.providedBy(portlet))
        self.failUnless(IPortletDataProvider.providedBy(portlet.data))

    def test_invoke_add_view(self):
        portlet = getUtility(IPortletType, name='ely.portlets.image.ImagePortlet')
        mapping = self.portal.restrictedTraverse('++contextportlets++plone.leftcolumn')
        for m in mapping.keys():
            del mapping[m]
        addview = mapping.restrictedTraverse('+/' + portlet.addview)
        addview.createAndAdd(data={'image':_test_image_object()})

        self.assertEquals(len(mapping), 1)
        self.failUnless(isinstance(mapping.values()[0], imageportlet.Assignment))

    def test_invoke_edit_view(self):
        mapping = PortletAssignmentMapping()
        request = self.folder.REQUEST

        mapping['foo'] = imageportlet.Assignment(**test_assignment_data())
        editview = getMultiAdapter((mapping['foo'], request), name='edit')
        self.failUnless(isinstance(editview, imageportlet.EditForm))

    def test_obtain_renderer(self):
        context = self.folder
        request = self.folder.REQUEST
        view = self.folder.restrictedTraverse('@@plone')
        manager = getUtility(IPortletManager, name='plone.rightcolumn', context=self.portal)

        assignment = imageportlet.Assignment(**test_assignment_data())
        renderer = getMultiAdapter((context, request, view, manager, assignment), IPortletRenderer)
        self.failUnless(isinstance(renderer, imageportlet.Renderer))

class TestRenderer(TestCase):

    def afterSetUp(self):
        self.setRoles(('Manager',))

    def renderer(self, context=None, request=None, view=None, manager=None, assignment=None):
        context = context or self.folder
        request = request or self.folder.REQUEST
        view = view or self.folder.restrictedTraverse('@@plone')
        manager = manager or getUtility(IPortletManager, name='plone.rightcolumn', context=self.portal)
        assignment = assignment or imageportlet.Assignment(**test_assignment_data())
        return getMultiAdapter((context, request, view, manager, assignment), IPortletRenderer)

    def test_render(self):
        renderer = self.renderer(
            context=self.portal,
            assignment=imageportlet.Assignment(**test_assignment_data()))
        renderer = renderer.__of__(self.folder)
        renderer.update()
        output = renderer.render()
        self.failUnless("<img src='http://nohost/plone/++contextportlets++plone.rightcolumn//@@image' width='200' height='260'/>" in output)
        self.failUnless('<a href="http://nohost/plone/front-page">' in output)
        self.failUnless('<span class="img-caption">some kind of image</span>' in output)
        self.failUnless('<span class="img-title">Test Image Portlet</span>' in output)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestPortlet))
    suite.addTest(makeSuite(TestRenderer))
    return suite
